/*
 * SPDX-FileCopyrightText: 2014 David Edmundson <david@davidedmundson.co.uk>
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 *
 */

#include "abstractnoteloader.h"

#include <QString>

AbstractNoteLoader::AbstractNoteLoader()
{
}

AbstractNoteLoader::~AbstractNoteLoader()
{
}
